#############################################################
# compute_risk_metrics.py
# or "13 Compute Risk Metrics for Segments" tool in the SICCM toolbox
#
# By Michael Bunn, Ben Leshchinsky, and Michael Olsen
# Oregon State University
# Last Updated: 11/28/2018
#
# Creation of this tool has been supported by Oregon Department of
# Transportation research project SPR 786 and
# United States Forest Service grant number 17-CS-11015600-008
#
# Direct questions to ben.leshchinsky@oregonstate.edu
#############################################################

# Import modules, initiate arcpy
import arcpy
import math
import os
from arcpy import env
from arcpy.sa import *
arcpy.env.overwriteOutput = True
arcpy.CheckOutExtension("Spatial")

# Get parameters from GUI
Risk_Zones = arcpy.GetParameterAsText(0)
idx = Risk_Zones.find("Segments")
if idx == -1:
	arcpy.AddError('At-Risk Segments are not a valid file')
idx = idx+9
DEMunits = Risk_Zones[idx:]

if DEMunits == "FootUS":
	DEMunits = "Foot"

Emb_Slope = arcpy.GetParameterAsText(1)
if Emb_Slope == '#' or not Emb_Slope:
	Emb_Slope = str(26)

Max_Width = arcpy.GetParameter(2)
if Max_Width == '#' or not Max_Width:
	Max_Width = 70

if DEMunits == "Foot":
	Max_Width = Max_Width
else:
	Max_Width = Max_Width/3.28
	
Max_Width = str(Max_Width)

Repair_Rate = arcpy.GetParameterAsText(3)
if Repair_Rate == '#' or not Repair_Rate:
	Repair_Rate = str(250)
	
Excav = arcpy.GetParameterAsText(4)
if Excav == '#' or not Excav:
	Excav = str(11.00)
	
Fill = arcpy.GetParameterAsText(5)
if Fill == '#' or not Fill:
	Fill = str(18.50)

# Convert repair rate to cubic feet
RepairR = float(Repair_Rate)*27

# Define new addField function
def addField(Features,Fieldname,Type):
	check = 0
	lst = arcpy.ListFields(Features)
	for f in lst:
		if f.name == Fieldname:
			check = 1
		else:
			continue
	if check == 0:
		arcpy.AddField_management(Features,Fieldname,Type)
	

#env.workspace = OutFolder

# Add new EMBSLOPE field
addField(Risk_Zones,"EMBSLOPE","DOUBLE")

# Compute EMBSLOPE field
arcpy.CalculateField_management(Risk_Zones, "EMBSLOPE", Emb_Slope, "VB","")

# Add new WIDTH field
addField(Risk_Zones,"WIDTH","DOUBLE")

# Compute WIDTH field
try:
	arcpy.CalculateField_management(Risk_Zones, "WIDTH", "!BLOCK_AREA!//!LENGTH!", "PYTHON")
except:
	arcpy.CalculateField_management(Risk_Zones, "WIDTH", "!BLOCK_AREA!//!LENGTH_GEO!", "PYTHON")
arcpy.MakeFeatureLayer_management(Risk_Zones,"RiskLyr")

# Find widths greater than Max_Width and change their value to Max_Width
expr = "\"WIDTH\" >= "+Max_Width
arcpy.SelectLayerByAttribute_management("RiskLyr","NEW_SELECTION",expr)
arcpy.CalculateField_management("RiskLyr", "WIDTH",Max_Width)
arcpy.SelectLayerByAttribute_management("RiskLyr", "CLEAR_SELECTION")

# Add new XS_AREA field - cross sectional area
addField("RiskLyr","XS_AREA","DOUBLE")
	
# Calculate XS_AREA FIELD
codeblock = """def calcArea(emWidth,lsSlope,emSlope):
	A = emWidth
	b = math.radians(180 - emSlope)
	c = math.radians(lsSlope)
	a = math.pi - b - c
	B = (A*math.sin(b))/(math.sin(a))
	C = (A*math.sin(c))/(math.sin(a))
	S = (A+B+C)/2
	if (S*(S-A)*(S-B)*(S-C)) >= 0:
		Aval = math.sqrt(S*(S-A)*(S-B)*(S-C))
	else:
		Aval = 0
	return Aval"""

expression = "calcArea(!WIDTH!,!LS_SLOPE!,!EMBSLOPE!)"

arcpy.CalculateField_management("RiskLyr", "XS_AREA", expression, "PYTHON", codeblock)

# Add new VOLUME field and compute volume
addField("RiskLyr","VOLUME","DOUBLE")
try:
	arcpy.CalculateField_management("RiskLyr", "VOLUME", "!XS_AREA!*!LENGTH!", "PYTHON")
except:
	arcpy.CalculateField_management("RiskLyr", "VOLUME", "!XS_AREA!*!LENGTH_GEO!", "PYTHON")

# Compute TIME_CLOSE
addField("RiskLyr","TIME_CLOSE","DOUBLE")
expression = "!VOLUME!*35.3147/"+str(RepairR)
arcpy.CalculateField_management("RiskLyr", "TIME_CLOSE", expression, "PYTHON")

# Compute Repair Cost
ExcavF = float(Excav)/27
FillF = float(Fill)/27
TotalCost = (ExcavF+FillF)

addField("RiskLyr","RP_COST","DOUBLE")
addField("RiskLyr","RP_COST_FT","DOUBLE")

expression = "!VOLUME!*35.3147*"+str(TotalCost)
arcpy.CalculateField_management("RiskLyr", "RP_COST", expression, "PYTHON")
try:
	arcpy.CalculateField_management("RiskLyr", "RP_COST_FT", "!RP_COST!/(!LENGTH!*3.28)", "PYTHON")
except:
	arcpy.CalculateField_management("RiskLyr", "RP_COST_FT", "!RP_COST!/(!LENGTH_GEO!*3.28)", "PYTHON")


arcpy.Delete_management("RiskLyr")



