#############################################################
# create_risk_map.py
# or "12 Create At-Risk Segments" tool in the SICCM toolbox
#
# By Michael Bunn, Ben Leshchinsky, and Michael Olsen
# Oregon State University
# Last Updated: 11/28/2018
#
# Creation of this tool has been supported by Oregon Department of
# Transportation research project SPR 786 and
# United States Forest Service grant number 17-CS-11015600-008
#
# Direct questions to ben.leshchinsky@oregonstate.edu
#############################################################

# Import modules, initiate arcpy
import os
import arcpy
from arcpy import env
from arcpy.sa import *
arcpy.env.overwriteOutput = True
arcpy.CheckOutExtension("Spatial")

# Get input strings from GUI
#1 - RS_Elev
RS_Elev = arcpy.GetParameterAsText(0)
#2 - Deposits from CCM
Deposits = arcpy.GetParameterAsText(1)
#3 - Highways
Highway = arcpy.GetParameterAsText(2)

# Find location of input
descInput = arcpy.Describe(Deposits)
arcpy.env.workspace = os.path.dirname(descInput.catalogPath)

spat = arcpy.Describe(RS_Elev).spatialReference
unit = spat.linearUnitName

# Display output location
loc = arcpy.env.workspace
commentval = "Outputs will be saved to "+loc
arcpy.AddMessage(commentval)

arcpy.AddMessage("Warning - this tool takes longer than most")


# Create temporary files
DisDep = "DisDep"
DisRoad = "DisRoad"
UniDep = "UniDep"
RS_Slope = "RS_Slope"
RC_Slope = "RC_Slope"
Ext_Slope = "Ext_Slope"
Ext_Poly = "Ext_Poly"
Blocks = "Blocks"
Zonal_Slope = "Zonal_Slope"
Block_Stats = "Block_Stats"
Risk_Zones = "Risk_Segments_"+unit
Risk_Zones_MP = "Risk_Zones_MP"

# Fix Geometry of Deposits
arcpy.RepairGeometry_management(Deposits)

arcpy.AddMessage("Initial Processes Complete (1/5)")

# Create DISS field and dissolve Deposits and Roads - DisDep, DisRoad
check = 0
lst = arcpy.ListFields(Deposits)
for f in lst:
	if f.name == "DIS":
		check = 1
	else:
		continue
if check == 0:
	arcpy.AddField_management(Deposits,"DIS","SHORT")
arcpy.Dissolve_management(Deposits, DisDep, "DIS")

check = 0
lst = arcpy.ListFields(Highway)
for f in lst:
	if f.name == "DIS":
		check = 1
	else:
		continue
if check == 0:
	arcpy.AddField_management(Highway,"DIS","SHORT")
arcpy.Dissolve_management(Highway, DisRoad, "DIS")

# Perform union on Deposits - UniDep
arcpy.Union_analysis(Deposits, UniDep)

arcpy.AddMessage("Feature Class Manipulation Complete (2/5)")

## PREPARE BLOCK POLYGONS

# Compute Slope - RS_Slope
outSlope = Slope(RS_Elev)
outSlope.save(RS_Slope)

# Reclassify Slope - RC_Slope
outReclass = Reclassify(RS_Slope, "VALUE", "0 5 1;5 30 2;30 90 3", "DATA")
outReclass.save(RC_Slope)

# Extract Slope by attributes - Ext_Slope
attExtract = ExtractByAttributes(RC_Slope, "\"VALUE\" = 1")
attExtract.save(Ext_Slope)

# Convert Extracted Slope to polygons - Ext_Poly
arcpy.RasterToPolygon_conversion(Ext_Slope, Ext_Poly, "SIMPLIFY")
  
# Clip Slope polygons using dissolved Deposits - Blocks
arcpy.Clip_analysis(Ext_Poly, DisDep, Blocks)



# Compute Area of remaining Slope polygons - 
arcpy.AddGeometryAttributes_management(Blocks, "AREA", "", "SQUARE_FEET_US")

arcpy.AddMessage("Unstable Blocks Have Been Identified (3/5)")

## COMPUTE BLOCK SLOPE

# Perform zonal statistics on RS_Slope with UniDep - Zonal_Slope
try:
	outZS = ZonalStatistics(UniDep, "FID", RS_Slope, "MEAN")
	outZS.save(Zonal_Slope)
except:
	outZS = ZonalStatistics(UniDep, "OBJECTID", RS_Slope, "MEAN")
	outZS.save(Zonal_Slope)

try:
	# Perform zonal stats as table on Zonal_Slope with Blocks using FID - Block_Stats
	outZSaT = ZonalStatisticsAsTable(Blocks, "FID", Zonal_Slope, Block_Stats, "", "MEAN")

	# Join Block_Stats with Blocks
	arcpy.MakeFeatureLayer_management(Blocks, "BlockLyr")
	arcpy.AddJoin_management("BlockLyr", "FID", Block_Stats, "FID")
except:
	# Perform zonal stats as table on Zonal_Slope with Blocks using FID - Block_Stats
	outZSaT = ZonalStatisticsAsTable(Blocks, "OBJECTID", Zonal_Slope, Block_Stats, "", "MEAN")

	# Join Block_Stats with Blocks
	arcpy.MakeFeatureLayer_management(Blocks, "BlockLyr")
	arcpy.AddJoin_management("BlockLyr", "OBJECTID", Block_Stats, "OBJECTID")	
arcpy.AddMessage("Slope Statistics Complete and Assigned to Unstable Blocks (4/5)")

## INTERSECT

# Intersect Blocks with DisRoad - Risk_Zones
inFeatures = [DisRoad,"BlockLyr"]
arcpy.Intersect_analysis(inFeatures, Risk_Zones_MP)

arcpy.MultipartToSinglepart_management(Risk_Zones_MP,Risk_Zones)

# Compute length of Risk_Zones
try:
	arcpy.AddGeometryAttributes_management(Risk_Zones, "LENGTH", "FEET_US", "")
except:
	arcpy.AddGeometryAttributes_management(Risk_Zones, "LENGTH_GEODESIC", "FEET_US", "")
# Delete Temporary Files
arcpy.Delete_management(DisDep)
arcpy.Delete_management(DisRoad)
arcpy.Delete_management(UniDep)
arcpy.Delete_management(RS_Slope)
arcpy.Delete_management(RC_Slope)
arcpy.Delete_management(Ext_Slope)
arcpy.Delete_management(Ext_Poly)
arcpy.Delete_management(Blocks)
arcpy.Delete_management(Zonal_Slope)
arcpy.Delete_management(Block_Stats)
arcpy.Delete_management(Block_Stats)
arcpy.Delete_management(outZSaT)
arcpy.Delete_management(outZS)
arcpy.Delete_management(Risk_Zones_MP)
arcpy.Delete_management("BlockLyr")

arcpy.AddMessage("Intersect complete (5/5)")

# Alter field names and delete unnecessary fields
def addField(Features,Fieldname,Type):
	check = 0
	lst = arcpy.ListFields(Features)
	for f in lst:
		if f.name == Fieldname:
			check = 1
		else:
			continue
	if check == 0:
		arcpy.AddField_management(Features,Fieldname,Type)

addField(Risk_Zones,"BLOCK_AREA","DOUBLE")
arcpy.CalculateField_management(Risk_Zones, "BLOCK_AREA", "!Blocks_POLY_AREA!", "PYTHON")

addField(Risk_Zones,"LS_SLOPE","DOUBLE")
arcpy.CalculateField_management(Risk_Zones, "LS_SLOPE", "!Block_Stats_MEAN!", "PYTHON")

arcpy.DeleteField_management(Risk_Zones,"FID_DisRoad")
arcpy.DeleteField_management(Risk_Zones,"DIS")
arcpy.DeleteField_management(Risk_Zones,"FID_Blocks_Block_Stats")
arcpy.DeleteField_management(Risk_Zones,"Blocks_POLY_AREA")
arcpy.DeleteField_management(Risk_Zones,"Blocks_gridcode")
arcpy.DeleteField_management(Risk_Zones,"Block_Stats_OBJECTID")
arcpy.DeleteField_management(Risk_Zones,"Block_Stats_OBJECTID_1")
arcpy.DeleteField_management(Risk_Zones,"Block_Stats_COUNT")
arcpy.DeleteField_management(Risk_Zones,"Block_Stats_AREA")
arcpy.DeleteField_management(Risk_Zones,"Block_Stats_MEAN")
arcpy.DeleteField_management(Risk_Zones,"ORIG_FID")

## Display segments on map

lyrDir = os.path.dirname(__file__)
fileSeg = lyrDir+"\\risksegment.lyr"

# Set the map document and data frame
mxd = arcpy.mapping.MapDocument("CURRENT")
df = arcpy.mapping.ListDataFrames(mxd,"Layers")[0]

# Create layer file and add to map
lyrSeg = arcpy.mapping.Layer(Risk_Zones)
arcpy.ApplySymbologyFromLayer_management(lyrSeg, fileSeg)
try:
	arcpy.mapping.RemoveLayer(df, arcpy.mapping.ListLayers(mxd, lyrSeg)[0])
	arcpy.mapping.AddLayer(df,lyrSeg)
except:
	arcpy.mapping.AddLayer(df,lyrSeg)


arcpy.CheckInExtension("spatial")



