##########################################################################
# digitize_streams.py
# or "06 Digitize Stream Channels" tool in the CCM Flow toolbox
#
# By Michael Bunn, Ben Leshchinsky, and Michael Olsen
# Oregon State University
# Last Updated: 07/11/2018
#
# Creation of this tool has been supported by Oregon Department of
# Transportation research project SPR 786 and
# United States Forest Service grant number 17-CS-11015600-008
#
# Direct questions to ben.leshchinsky@oregonstate.edu
##########################################################################

# Import arcpy module
import arcpy, os
from arcpy.sa import *
arcpy.env.overwriteOutput=True
arcpy.CheckOutExtension("spatial")

# Define conversion function##########################################
def ArealUnit2Pixel(areal_unit,cell_size,raster_units):
	# Determine the area of an individual DEM cell
	# Units are in raster_units squared
	cell_area = cell_size*cell_size
	
	# Break Areal Unit into two parts; unit and value
	idx = areal_unit.find(" ")
	idx1 = int(idx+1)
	unit = areal_unit[idx1:]
	accval = float(areal_unit[:idx])
	
	# Create a dictionary with conversion values
	conversions = {"SquareFeet":[1,0.092903], \
	"Ares":[1076.39,100], \
	"Acres":[43560,4046.86], \
	"Hectares":[107639,10000], \
	"SquareCentimeters":[0.00107639,0.0001], \
	"SquareDecimeters":[0.107639,0.01], \
	"SquareInches":[0.00694444,0.00064516], \
	"SquareKilometers":[10760000,1000000], \
	"SquareMeters":[10.7639,0], \
	"SquareMiles":[27880000,2590000], \
	"SquareMillimeters":[0.0000107639,0.000001], \
	"SquareYards":[9,0.836127], \
	"Pixels":[1,1]}
	
	# Mutliply the user specified area by a conversion to
	# Square Feet or Square Meters
	if raster_units == "Meter":
		# Accumulated area in square meters
		user_area = accval*conversions[unit][1]
	elif raster_units == "Foot":
		# Accumulated area in square feet
		user_area = accval*conversions[unit][0]
	
	# Divide user specified area by cell area to get number of pixels
	numpix = user_area/cell_area
	if unit == "Pixels":
		numpix = user_area
	return numpix, user_area
######################################################################


# Ask user for input DEM and its units
inputDEM = arcpy.GetParameterAsText(0)

# Find location of input
descInput = arcpy.Describe(inputDEM)
arcpy.env.workspace = os.path.dirname(descInput.catalogPath)

# Display output location
loc = arcpy.env.workspace
commentval = "Outputs will be saved to "+loc
arcpy.AddMessage(commentval)

# Determine cellsize and unit of input DEM
description = arcpy.Describe(inputDEM)
cellsize = description.children[0].meanCellHeight

spat = arcpy.Describe(inputDEM).spatialReference
DEMunits = spat.linearUnitName

if DEMunits == "FootUS":
	DEMunits = "Foot"

# Ask user if they would like to perform default first run
RunDefaults = arcpy.GetParameterAsText(1)

#### Follow typical stream identification workflow in ArcGIS #####

# Assign temporary variables
FillDEM = "FillDEM"
FlowDir = "FlowDir"
DropRast = "DropRast"
FlowAcc = "FlowAcc_"+str(int(round(cellsize)))+DEMunits
ReclassRast = "ReclassRast"


# Fill input DEM
arcpy.gp.Fill_sa(inputDEM, FillDEM, "")

# Compute flow direction from filled DEM
arcpy.gp.FlowDirection_sa(FillDEM, FlowDir, "NORMAL", DropRast)

# Compute flow accumulation raster
arcpy.gp.FlowAccumulation_sa(FlowDir, FlowAcc, "", "FLOAT")

##################################################################

### BEGIN DISPLAY PREPARATIONS ###

# Set the map document and data frame
mxd = arcpy.mapping.MapDocument("CURRENT")
df = arcpy.mapping.ListDataFrames(mxd,"Layers")[0]

# Find location of layer file
lyrDir = os.path.dirname(__file__)
fileStreams = lyrDir+"\\streams.lyr"
fileAcc = lyrDir+"\\flowacc.lyr"

### END DISPLAY PREPARATIONS ###


# Compute first run, if desired
if RunDefaults == "true":
	
	# Loop through default values
	DefUnits = "Acres"
	DefAcc = [1, 3, 5]
	for AccVal in DefAcc:
		# Compute pixel counts and areas
		areal = str(AccVal)+" "+DefUnits
		pix, area = ArealUnit2Pixel(areal,cellsize, DEMunits)
		# Prepare variable names for computations
		strVal = str(round(AccVal,2))
		strVal = strVal.replace(".","_")
		Streams = "Channels_"+strVal+DefUnits
		remap = RemapRange([[0, pix, 0], [pix, 1000000000, 1]])
		
		# Reclassify flow accumulation raster
		OutReclass = Reclassify(FlowAcc, "VALUE", remap, "DATA")
		OutReclass.save(ReclassRast)
		
		# Convert reclassified accumulation raster into lines
		arcpy.RasterToPolyline_conversion(ReclassRast, Streams, "ZERO", "0", "SIMPLIFY", "VALUE")
		
		# Apply symbology from layer file
		lyrStreams = arcpy.mapping.Layer(Streams)
		arcpy.ApplySymbologyFromLayer_management(lyrStreams, fileStreams)
		try:
			arcpy.mapping.RemoveLayer(df, arcpy.mapping.ListLayers(mxd, lyrStreams)[0])
			arcpy.mapping.AddLayer(df,lyrStreams)
		except:
			arcpy.mapping.AddLayer(df,lyrStreams)
		
# Get custom accumulation unit
# Input is list: "Square Feet", "Acres", "Ares", "Hectares", "Square Kilometers",
# "Square Miles", "Square Meters", "Square Yards"
CustomUnitsSpace = arcpy.GetParameterAsText(2)

# Remove spaces from custom unit name
CustomUnits = CustomUnitsSpace.replace(" ","")

# Get custom accumulation values
CustomAcc = arcpy.GetParameter(3)

for AccVal in CustomAcc:
		# Compute pixel counts and areas
		areal = str(AccVal)+" "+CustomUnits
		pix, area = ArealUnit2Pixel(areal,cellsize, DEMunits)
		
		# Prepare variable names for computations
		strVal = str(round(AccVal,2))
		strVal = strVal.replace(".","_")
		Streams = "Channels_"+strVal+CustomUnits
		remap = RemapRange([[0, pix, 0], [pix, 1000000000, 1]])
		
		# Reclassify flow accumulation raster
		OutReclass = Reclassify(FlowAcc, "VALUE", remap, "DATA")
		OutReclass.save(ReclassRast)
		
		# Convert reclassified accumulation raster into lines
		arcpy.RasterToPolyline_conversion(ReclassRast, Streams, "ZERO", "0", "SIMPLIFY", "VALUE")
		
		# Apply symbology from layer file
		lyrStreams = arcpy.mapping.Layer(Streams)
		arcpy.ApplySymbologyFromLayer_management(lyrStreams, fileStreams)
		arcpy.mapping.AddLayer(df,lyrStreams)

# Apply symbology from layer file
lyrAcc = arcpy.mapping.Layer(FlowAcc)
arcpy.ApplySymbologyFromLayer_management(lyrAcc, fileAcc)
try:
	arcpy.mapping.RemoveLayer(df, arcpy.mapping.ListLayers(mxd, lyrAcc)[0])
	arcpy.mapping.AddLayer(df,lyrAcc)
except:
	arcpy.mapping.AddLayer(df,lyrAcc)
	
# Delete temporary files
arcpy.Delete_management(FillDEM)
arcpy.Delete_management(FlowDir)
arcpy.Delete_management(DropRast)
arcpy.Delete_management(ReclassRast)

arcpy.CheckInExtension("spatial")