###################################################################
# prepare_mixture_raster.py
# or "04 Create Mixture Raster" tool in the SICCM toolbox
#
# By Michael Bunn, Ben Leshchinsky, and Michael Olsen
# Oregon State University
# Last Updated: 07/11/2018
#
# Creation of this tool has been supported by Oregon Department of
# Transportation research project SPR 786 and
# United States Forest Service grant number 17-CS-11015600-008
#
# Direct questions to ben.leshchinsky@oregonstate.edu
###################################################################

# Import arcpy module
import arcpy, os
from arcpy.sa import *
arcpy.env.overwriteOutput=True

# Check out any necessary licenses
arcpy.CheckOutExtension("spatial")

# Ask user for input DEM
input_raster = arcpy.GetParameterAsText(0)

# Find location of input
descInput = arcpy.Describe(input_raster)
arcpy.env.workspace = os.path.dirname(descInput.catalogPath)

# Display output location
loc = arcpy.env.workspace
commentval = "Outputs will be saved to "+loc
arcpy.AddMessage(commentval)

# Ask user for best slope raster
BestSlope = arcpy.GetParameterAsText(1)

# Determine cellsize and unit of best slope raster
description = arcpy.Describe(BestSlope)
cellsize = description.children[0].meanCellHeight

spat = arcpy.Describe(BestSlope).spatialReference
unit = spat.linearUnitName
	
res = str(int(round(cellsize)))
Output_Cell_Size = res+" "+res

# Name mixture file that will be output "Mix_CSXX" Cell Size XX
OutputMixture = "Mix_CS"+res+unit

# Name files to be output
OutputRSelev = "Resampled_DEM"+res+unit

# Name of local files to be deleted
OutputSlope = "SlopeTemp"
OutputProfile = "ProfTemp"
OutputPlan = "PlanTemp"
SmoothInputDEM = "SmDEMTemp"
Curva = "CurvaTemp"
MixUnsmooth = "UnsmMixTemp"

# Perform smoothing on input DEM
arcpy.gp.FocalStatistics_sa(input_raster, SmoothInputDEM, "Rectangle 5 5 CELL", "MEAN", "DATA")

# Resample smoothed input DEM to selected cellsize
arcpy.Resample_management(SmoothInputDEM, OutputRSelev, Output_Cell_Size, "BILINEAR")

# Compute slope and profile curvature from resampled DEM
outSlope = Slope(OutputRSelev, "DEGREE", "1")
outSlope.save(OutputSlope)

outCurve = Curvature(OutputRSelev, "1", OutputProfile, OutputPlan)
outCurve.save(Curva)
	
	
# Multiply slope and profile curvature together to obtain
# unsmoothed mixture raster
outTimes = Times(OutputSlope, OutputProfile)
outTimes.save(MixUnsmooth)

# Smooth mixture raster
arcpy.gp.FocalStatistics_sa(MixUnsmooth, OutputMixture, "Rectangle 5 5 CELL", "MEAN", "DATA")

# Delete local files
arcpy.Delete_management(OutputSlope, "")
arcpy.Delete_management(OutputProfile, "")
arcpy.Delete_management(OutputPlan, "")
arcpy.Delete_management(SmoothInputDEM, "")
arcpy.Delete_management(Curva, "")
arcpy.Delete_management(MixUnsmooth, "")

## Display mixture raster on map

# Set the map document and data frame
mxd = arcpy.mapping.MapDocument("CURRENT")
df = arcpy.mapping.ListDataFrames(mxd,"Layers")[0]

lyrRS = arcpy.mapping.Layer(OutputRSelev)
lyrRS.transparency = 100
arcpy.mapping.AddLayer(df,lyrRS)

lyrDir = os.path.dirname(__file__)
fileMix = lyrDir+"\\mixture.lyr"

lyrMix = arcpy.mapping.Layer(OutputMixture)
arcpy.ApplySymbologyFromLayer_management(lyrMix, fileMix)
arcpy.mapping.AddLayer(df,lyrMix)

arcpy.CheckInExtension("spatial")
